@@ -5,12 +5,12 @@ require 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '3.52';
+our $VERSION = '3.53';
 $VERSION = eval $VERSION;
 
-use Object::InsideOut::Exception 3.52;
-use Object::InsideOut::Util 3.52 qw(create_object hash_re is_it make_shared);
-use Object::InsideOut::Metadata 3.52;
+use Object::InsideOut::Exception 3.53;
+use Object::InsideOut::Util 3.53 qw(create_object hash_re is_it make_shared);
+use Object::InsideOut::Metadata 3.53;
 
 require B;
 
@@ -100,6 +100,8 @@ if (! exists($GBL{'GBL_SET'})) {
             obj => make_shared({}),
             ok  => $threads::shared::threads_shared,
         },
+
+        # cache                 # Object initialization activity cache
     );
 
     # Add metadata
@@ -670,7 +672,7 @@ sub _ID :Sub
 
 ### Initialization Handling ###
 
-# Finds a subroutine's name from itsft(@{$$reuse{$tree}[$thread_id code ref
+# Finds a subroutine's name from its code ref
 sub sub_name :Sub(Private)
 {
     my ($ref, $attr, $location) = @_;
@@ -1541,13 +1543,25 @@ sub new :MergeArgs
     # Create a new 'bare' object
     my $self = _obj($class);
 
+    # Object initialization activity caching
+    my $have_cache = exists($GBL{'cache'}{$class});
+    my %cache = ($have_cache) ? %{$GBL{'cache'}{$class}}
+                              : ( 'pre'  => 0, 'def'  => 0 );
+
     # Execute pre-initialization subroutines
-    my $preinit_subs = $GBL{'sub'}{'pre'};
-    if (%{$preinit_subs}) {
-        foreach my $pkg (@{$GBL{'tree'}{'bu'}{$class}}) {
-            if (my $preinit = $$preinit_subs{$pkg}) {
-                local $SIG{'__DIE__'} = 'OIO::trap';
-                $self->$preinit($all_args);
+    if ($cache{'pre'} || ! $have_cache) {
+        my $preinit_subs = $GBL{'sub'}{'pre'};
+        if (%{$preinit_subs}) {
+            foreach my $pkg (@{$GBL{'tree'}{'bu'}{$class}}) {
+                if (my $preinit = $$preinit_subs{$pkg}) {
+                    local $SIG{'__DIE__'} = 'OIO::trap';
+                    $self->$preinit($all_args);
+                    if ($have_cache) {
+                        last if (! (--$cache{'pre'}));
+                    } else {
+                        $cache{'pre'}++;
+                    }
+                }
             }
         }
     }
@@ -1555,10 +1569,17 @@ sub new :MergeArgs
     my $tree = $GBL{'tree'}{'td'}{$class};
 
     # Set any defaults
-    foreach my $pkg (@{$tree}) {
-        if (my $def = $GBL{'fld'}{'def'}{$pkg}) {
-            $self->set($_->[0], Object::InsideOut::Util::clone($_->[1]))
-                foreach (@{$def});
+    if ($cache{'def'} || ! $have_cache) {
+        foreach my $pkg (@{$tree}) {
+            if (my $def = $GBL{'fld'}{'def'}{$pkg}) {
+                $self->set($_->[0], Object::InsideOut::Util::clone($_->[1]))
+                    foreach (@{$def});
+                if ($have_cache) {
+                    last if (! (--$cache{'def'}));
+                } else {
+                    $cache{'def'}++;
+                }
+            }
         }
     }
 
@@ -1624,6 +1645,11 @@ sub new :MergeArgs
         }
     }
 
+    # Remember object initialization activity caching
+    if (! $have_cache) {
+        $GBL{'cache'}{$class} = \%cache;
+    }
+
     # Done - return object
     return ($self);
 }
@@ -1947,7 +1973,14 @@ sub STORABLE_thaw :Sub
     }
 
     # Recreate the object
-    my $self = Object::InsideOut->pump($data);
+    my $self;
+    eval {
+        $self = Object::InsideOut->pump($data);
+    };
+    if ($@) {
+        die($@->as_string());   # Storable doesn't like exception objects
+    }
+
     # Transfer the ID to Storable's object
     $$obj = $$self;
     # Make object shared, if applicable
